==> ejc-proequipmentcorp-impl-3.0.0-20221017.101726-8-sources.jar: com/ejianc/business/proequipmentcorppur/purchase/service/impl/PurchaseContractBpmServiceImpl.java <==
package com.ejianc.business.proequipmentcorppur.purchase.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ejianc.business.contractbase.filing.enums.FilingStatusEnum;
import com.ejianc.business.proequipmentcorppur.purchase.bean.PurchaseContractEntity;
import com.ejianc.business.proequipmentcorppur.purchase.enums.DraftTypeEnum;
import com.ejianc.business.proequipmentcorppur.purchase.enums.PerformanceStatusEnum;
import com.ejianc.business.proequipmentcorppur.purchase.enums.SignatureStatusEnum;
import com.ejianc.business.proequipmentcorppur.purchase.service.IContractAsyncService;
import com.ejianc.business.proequipmentcorppur.purchase.service.IPurchaseContractService;
import com.ejianc.business.proequipmentcorppur.purchase.vo.PurchaseContractVO;
import com.ejianc.business.signaturemanage.api.ISignatureCommonApi;
import com.ejianc.business.signaturemanage.vo.WatermarkVO;
import com.ejianc.foundation.support.api.IBillTypeApi;
import com.ejianc.foundation.support.api.IParamConfigApi;
import com.ejianc.foundation.support.vo.ParamRegisterSetVO;
import com.ejianc.framework.auth.session.SessionManager;
import com.ejianc.framework.auth.session.UserContext;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.skeleton.billState.service.ICommonBusinessService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;

@Service("purchaseContract")
public class PurchaseContractBpmServiceImpl implements ICommonBusinessService {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IPurchaseContractService contractService;

    @Autowired
    private IBillTypeApi billTypeApi;

    @Autowired
    private SessionManager sessionManager;

    @Autowired
    private IContractAsyncService purchaseContractAsyncService;

    private static final String WATERMARK_CHECK_PARAM_NAME = "P-00a9W886";
    @Autowired
    ISignatureCommonApi signatureCommonApi;
    @Autowired
    private IParamConfigApi paramConfigApi;

    /**
     * 提交前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeSubmitProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return CommonResponse.success();
    }

    ;

    /**
     * 提交完回调
     *
     * @param
     * @return
     */
    @Override
    public CommonResponse<String> afterSubmitProcessor(Long billId, Integer state, String billTypeCode) {
        PurchaseContractEntity e = contractService.selectById(billId);

        if (DraftTypeEnum.线上起草.getCode().toString().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().toString().equals(e.getDraftType())) {
            if (null == e.getContractFileId()) {
                //确保线上起草方式时，用户已编辑合同文件
                return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
            }
            if (!e.getContractFileSyncFlag()) {
                return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
            }
        }


        UserContext userContext = sessionManager.getUserContext();
        e.setCommitDate(new Date());
        e.setCommitUserCode(userContext.getUserCode());
        e.setCommitUserName(userContext.getUserName());
        contractService.saveOrUpdate(e, false);

        return CommonResponse.success("合同审批回调处理成功！");
    }

    /**
     * 有审批流的撤回前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeHasBpmBack(Long billId, Integer state, String billTypeCode) {
        return bpmBackCheck(billId, state, billTypeCode);
    }

    ;

    /**
     * 有审批流的撤回后回调
     *
     * @param
     * @return
     */
    @Override
    public CommonResponse<String> afterHasBpmBack(Long billId, Integer state, String billTypeCode) {
        return CommonResponse.success();
    }

    ;

    /**
     * 审批节点审批中时节点审批前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeInApprovalBack(Long billId, Integer state, String billTypeCode, String sign) {
        PurchaseContractEntity e = contractService.selectById(billId);
        logger.info("审批节点审批中时节点审批前回调:  当前数据-{}", JSONObject.toJSONString(e));
        //线上起草且文件未同步
        if ((DraftTypeEnum.线上起草.getCode().toString().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().toString().equals(e.getDraftType())) && !e.getContractFileSyncFlag()) {
            return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
        }

        return CommonResponse.success("审批处理成功！");
    }

    ;

    /**
     * 终审审核前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeApprovalProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return CommonResponse.success();
    }

    /**
     * 终审审核完回调
     *
     * @param
     * @return
     */
    @Override
    public CommonResponse<String> afterApprovalProcessor(Long billId, Integer state, String billTypeCode) {
				/*
			起草方式为线下签订则不需要签章
			当不使用电子签章时：创建时默认为未签章，单据审批通过后自动更改为【已签章】
			当签章状态变为【已签章】时，合同状态变为【履约中】
		 */
        PurchaseContractEntity contractEntity = contractService.selectById(billId);
        if (contractEntity != null) {

            if ((DraftTypeEnum.线下签订.getCode().toString()).equals(contractEntity.getDraftType())) {
                //线下签订合同无需进行签章操作，则审批通过后直接置为已生效状态
                logger.info("合同id-{}为线下签订直接修改为签章状态并设置为履约中", billId);
                contractEntity.setSignatureStatus(SignatureStatusEnum.已签章.getCode());
                contractEntity.setContractPerformanceState(PerformanceStatusEnum.履约中.getCode());
                //合同已生效，添加生效时间
                contractEntity.setEffectiveDate(new Date());
            }
            else if ((DraftTypeEnum.线上起草不使用电子签章.getCode().toString()).equals(contractEntity.getDraftType()) && contractEntity.getContractFileSyncFlag()) {
                logger.info("合同id-{}为线下签订直接修改为签章状态并设置为履约中", billId);
                contractEntity.setSignatureStatus(SignatureStatusEnum.已签章.getCode());
                contractEntity.setContractPerformanceState(PerformanceStatusEnum.履约中.getCode());
                //合同已生效，添加生效时间
                contractEntity.setEffectiveDate(new Date());
            }
            else if (DraftTypeEnum.线上起草.getCode().toString().equals(contractEntity.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().toString().equals(contractEntity.getDraftType())) {
                if (null == contractEntity.getContractFileId()) {
                    //确保线上起草方式时，用户已编辑合同文件
                    return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
                }
                if (!contractEntity.getContractFileSyncFlag()) {
                    return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
                }
            }

        }
//        // 合同添加水印
//        WatermarkVO watermarkVO = new WatermarkVO();
//        watermarkVO.setScheme("QI_YUE_SUO");
//        watermarkVO.setFileId(contractEntity.getContractFileId());
//        watermarkVO.setBillId(contractEntity.getId());
//        watermarkVO.setBillType("BT220323000000001");
//        watermarkVO.setSourceType("purchaseContract");
//
//        ArrayList<WatermarkConfigVO> watermarks = new ArrayList<>();
//        WatermarkConfigVO vo = new WatermarkConfigVO();
//        vo.setType("TEXT");
//        vo.setContent("北京益企联科技有限公司");
//        vo.setImageBase64("");
//        vo.setFontSize(48);
//        vo.setColor("#999999");
//        vo.setRotateAngle(30.0D);
//        vo.setTransparency(0.5D);
//        vo.setLocation("MIDDLE_CENTER");
//        watermarks.add(vo);
//        watermarkVO.setWatermarkConfigs(watermarks);
//
//        // 获取上下文并异步调用添加水印
//        HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
//        HashMap<String, String> headers = new HashMap<>();
//        headers.put("authority", request.getHeader("authority"));
//        headers.put("ejc-token", request.getHeader("ejc-token"));
//        contractAsyncService.fetchWatermarkAttachment(headers, watermarkVO);


        if (state.equals(BillStateEnum.COMMITED_STATE.getBillStateCode())) {
            //直审时添加单据提交人等信息
            contractEntity.setCommitDate(new Date());
            contractEntity.setCommitUserCode(sessionManager.getUserContext().getUserCode());
            contractEntity.setCommitUserName(sessionManager.getUserContext().getUserName());
        }
        // 水印系统参数
        CommonResponse<ParamRegisterSetVO> response = paramConfigApi.getByCode(WATERMARK_CHECK_PARAM_NAME);
        if (!response.isSuccess() || response.getData() == null) {
            throw new BusinessException("获取水印系统参数请求失败，失败原因：" + response.getMsg());
        }
        String valueData = response.getData().getValueData();
        Assert.hasText(valueData, "获取的水印系统参数不能为空!");

        // 是否限制： 0:不限制，1:限制
        if ("0".equals(valueData)) {
            // 转换水印参数配置：默认规则
            WatermarkVO watermarkVO = signatureCommonApi.fetchWatermarkConfig(contractEntity.getContractFileId(), contractEntity.getId(),
                    contractEntity.getBillCode(), "BT220323000000001", "purchaseContract");
            Assert.notNull(watermarkVO, "获取水印系统参数失败!");
            // 获取上下文并异步调用添加水印
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            HashMap<String, String> headers = new HashMap<>();
            headers.put("authority", request.getHeader("authority"));
            headers.put("ejc-token", request.getHeader("ejc-token"));
            purchaseContractAsyncService.fetchWatermarkAttachment(headers, watermarkVO);
        }


        contractService.saveOrUpdate(contractEntity);
        // 推送项目池
        contractService.pushContract(BeanMapper.map(contractEntity, PurchaseContractVO.class));
        return CommonResponse.success("合同审批回调成功！");
    }

    /**
     * 弃审前事件回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeAbstainingProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return bpmBackCheck(billId, state, billTypeCode);
    }

    /**
     * 弃审后事件回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> afterAbstainingProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return CommonResponse.success();
    }

    private CommonResponse<String> bpmBackCheck(Long billId, Integer state, String billTypeCode) {
		/*
			弃审逻辑：
			1、审批过程中，支持后台弃审；
			2、当审批通过，且没有走签章流程且没有下游业务时，支持后台弃审；
		 */
        PurchaseContractEntity contractEntity = contractService.selectById(billId);

        // 审批可以撤回的签章状态集合
        ArrayList<String> statusList = new ArrayList<>();
        statusList.add(SignatureStatusEnum.未签章.getCode());
        statusList.add(SignatureStatusEnum.已退回.getCode());
        statusList.add(SignatureStatusEnum.已撤回.getCode());
        statusList.add(SignatureStatusEnum.已过期.getCode());

        //使用电子签章且已有签章流程则不允许撤回
        if (DraftTypeEnum.线上起草.getCode().toString().equals(contractEntity.getDraftType())
                || DraftTypeEnum.上传合同.getCode().toString().equals(contractEntity.getDraftType())) {
            if (!statusList.contains(contractEntity.getSignatureStatus())) {
                return CommonResponse.error("当前单据已有签章流程，不能撤回！");
            }
        }
        else if (SignatureStatusEnum.已签章.getCode().equals(contractEntity.getSignatureStatus())) {
            contractEntity.setSignatureStatus(SignatureStatusEnum.未签章.getCode());
            contractEntity.setContractPerformanceState(PerformanceStatusEnum.未签订.getCode());
        }

        contractService.saveOrUpdate(contractEntity);
        //合同是否被其他单据引用
        CommonResponse<String> res = billTypeApi.checkQuote(billTypeCode, billId);
        logger.info("平台返回查询被引用情况" + res.isSuccess() + "----" + res.getMsg());

        if (!res.isSuccess()) {
            return CommonResponse.error("当前单据已被下游业务引用，不能弃审！");
        }

        // 删除文件中心水印文件
        if(contractEntity.getWatermarkContractFileId() != null) {
            boolean delWatermarkflag = contractService.delWatermarkContractFile(contractEntity.getWatermarkContractFileId());
            if (delWatermarkflag) {
                contractEntity.setWatermarkContractFileId(null);
            }
        }
        contractEntity.setContractPerformanceState(PerformanceStatusEnum.未签订.getCode());
        contractEntity.setFilingStatus(FilingStatusEnum.未归档.getTypeCode());
        contractService.saveOrUpdate(contractEntity, false);

        return CommonResponse.success("弃审/撤回前校验回调成功！");
    }

}

==> ejc-proequipmentcorp-impl-3.0.0-20230223.025151-333-sources.jar: com/ejianc/business/proequipmentcorppur/purchase/service/impl/PurchaseContractBpmServiceImpl.java <==
package com.ejianc.business.proequipmentcorppur.purchase.service.impl;

import com.alibaba.fastjson.JSONObject;
import com.ejianc.business.contractbase.filing.enums.FilingStatusEnum;
import com.ejianc.business.contractbase.pool.enums.ContractTypeEnum;
import com.ejianc.business.proequipmentcorppur.constants.PurCommonBillTypeEnum;
import com.ejianc.business.proequipmentcorppur.purchase.bean.PurchaseContractEntity;
import com.ejianc.business.proequipmentcorppur.purchase.enums.DraftTypeEnum;
import com.ejianc.business.proequipmentcorppur.purchase.enums.PerformanceStatusEnum;
import com.ejianc.business.proequipmentcorppur.purchase.enums.SignatureStatusEnum;
import com.ejianc.business.proequipmentcorppur.purchase.service.IContractAsyncService;
import com.ejianc.business.proequipmentcorppur.purchase.service.IPurchaseContractService;
import com.ejianc.business.proequipmentcorppur.purchase.vo.PurchaseContractVO;
import com.ejianc.business.proequipmentcorprent.ac.bean.ContractEntity;
import com.ejianc.business.prosub.enums.ProsubBillTypeEnum;
import com.ejianc.business.signaturemanage.api.ISignatureCommonApi;
import com.ejianc.business.signaturemanage.vo.WatermarkVO;
import com.ejianc.foundation.file.api.IAttachmentApi;
import com.ejianc.foundation.file.vo.AttachmentVO;
import com.ejianc.foundation.support.api.IBillTypeApi;
import com.ejianc.foundation.support.api.IParamConfigApi;
import com.ejianc.foundation.support.vo.ParamRegisterSetVO;
import com.ejianc.framework.auth.session.SessionManager;
import com.ejianc.framework.auth.session.UserContext;
import com.ejianc.framework.core.exception.BusinessException;
import com.ejianc.framework.core.kit.mapper.BeanMapper;
import com.ejianc.framework.core.response.BillStateEnum;
import com.ejianc.framework.core.response.CommonResponse;
import com.ejianc.framework.skeleton.billState.service.ICommonBusinessService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.Assert;
import org.springframework.util.CollectionUtils;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;

@Service("purchaseContract")
public class PurchaseContractBpmServiceImpl implements ICommonBusinessService {

    private Logger logger = LoggerFactory.getLogger(this.getClass());

    @Autowired
    private IAttachmentApi attachmentApi;

    @Autowired
    private IPurchaseContractService contractService;

    @Autowired
    private IBillTypeApi billTypeApi;

    @Autowired
    private SessionManager sessionManager;

    @Autowired
    private IContractAsyncService purchaseContractAsyncService;

    private static final String WATERMARK_CHECK_PARAM_NAME = "P-00a9W886";
    @Autowired
    ISignatureCommonApi signatureCommonApi;
    @Autowired
    private IParamConfigApi paramConfigApi;

    /**
     * 提交前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeSubmitProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return CommonResponse.success();
    }

    ;

    /**
     * 提交完回调
     *
     * @param
     * @return
     */
    @Override
    public CommonResponse<String> afterSubmitProcessor(Long billId, Integer state, String billTypeCode) {
        PurchaseContractEntity e = contractService.selectById(billId);

        if (DraftTypeEnum.线上起草.getCode().toString().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().toString().equals(e.getDraftType())) {
            if (null == e.getContractFileId()) {
                //确保线上起草方式时，用户已编辑合同文件
                return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
            }
            if (!e.getContractFileSyncFlag()) {
                return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
            }
        }


        UserContext userContext = sessionManager.getUserContext();
        e.setCommitDate(new Date());
        e.setCommitUserCode(userContext.getUserCode());
        e.setCommitUserName(userContext.getUserName());
        contractService.saveOrUpdate(e, false);

        return CommonResponse.success("合同审批回调处理成功！");
    }

    /**
     * 有审批流的撤回前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeHasBpmBack(Long billId, Integer state, String billTypeCode) {
        return bpmBackCheck(billId, state, billTypeCode);
    }

    ;

    /**
     * 有审批流的撤回后回调
     *
     * @param
     * @return
     */
    @Override
    public CommonResponse<String> afterHasBpmBack(Long billId, Integer state, String billTypeCode) {
        return CommonResponse.success();
    }

    ;

    /**
     * 审批节点审批中时节点审批前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeInApprovalBack(Long billId, Integer state, String billTypeCode, String sign) {
        PurchaseContractEntity e = contractService.selectById(billId);
        logger.info("审批节点审批中时节点审批前回调:  当前数据-{}", JSONObject.toJSONString(e));
        //线上起草且文件未同步
        if ((DraftTypeEnum.线上起草.getCode().toString().equals(e.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().toString().equals(e.getDraftType())) && !e.getContractFileSyncFlag()) {
            return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
        }

        return CommonResponse.success("审批处理成功！");
    }

    ;

    /**
     * 终审审核前回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeApprovalProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return CommonResponse.success();
    }

    /**
     * 终审审核完回调
     *
     * @param
     * @return
     */
    @Override
    public CommonResponse<String> afterApprovalProcessor(Long billId, Integer state, String billTypeCode) {
				/*
			起草方式为线下签订则不需要签章
			当不使用电子签章时：创建时默认为未签章，单据审批通过后自动更改为【已签章】
			当签章状态变为【已签章】时，合同状态变为【履约中】
		 */
        PurchaseContractEntity contractEntity = contractService.selectById(billId);
        if (contractEntity != null) {

            if ((DraftTypeEnum.线下签订.getCode().toString()).equals(contractEntity.getDraftType())) {
                //线下签订合同无需进行签章操作，则审批通过后直接置为已生效状态
                logger.info("合同id-{}为线下签订直接修改为签章状态并设置为履约中", billId);
                contractEntity.setSignatureStatus(SignatureStatusEnum.已签章.getCode());
                contractEntity.setContractPerformanceState(PerformanceStatusEnum.履约中.getCode());
                //合同已生效，添加生效时间
                contractEntity.setEffectiveDate(new Date());
            }
            else if ((DraftTypeEnum.线上起草不使用电子签章.getCode().toString()).equals(contractEntity.getDraftType()) && contractEntity.getContractFileSyncFlag()) {
                logger.info("合同id-{}为线下签订直接修改为签章状态并设置为履约中", billId);
                contractEntity.setSignatureStatus(SignatureStatusEnum.已签章.getCode());
                contractEntity.setContractPerformanceState(PerformanceStatusEnum.履约中.getCode());
                //合同已生效，添加生效时间
                contractEntity.setEffectiveDate(new Date());
            }
            else if (DraftTypeEnum.线上起草.getCode().toString().equals(contractEntity.getDraftType()) || DraftTypeEnum.线上起草不使用电子签章.getCode().toString().equals(contractEntity.getDraftType())) {
                if (null == contractEntity.getContractFileId()) {
                    //确保线上起草方式时，用户已编辑合同文件
                    return CommonResponse.error("请编辑合同文件并保存后再执行此操作！");
                }
                if (!contractEntity.getContractFileSyncFlag()) {
                    return CommonResponse.error("请重新进行合同编辑，同步单据信息到合同文件！");
                }
            }

        }

        //合同文件检查
        checkContractFile(contractEntity);

        if (state.equals(BillStateEnum.COMMITED_STATE.getBillStateCode())) {
            //直审时添加单据提交人等信息
            contractEntity.setCommitDate(new Date());
            contractEntity.setCommitUserCode(sessionManager.getUserContext().getUserCode());
            contractEntity.setCommitUserName(sessionManager.getUserContext().getUserName());
        }
        // 水印系统参数
        CommonResponse<ParamRegisterSetVO> response = paramConfigApi.getByCode(WATERMARK_CHECK_PARAM_NAME);
        if (!response.isSuccess() || response.getData() == null) {
            throw new BusinessException("获取水印系统参数请求失败，失败原因：" + response.getMsg());
        }
        String valueData = response.getData().getValueData();
        Assert.hasText(valueData, "获取的水印系统参数不能为空!");

        // 是否限制： 0:不限制，1:限制
        if ("0".equals(valueData)) {
            // 转换水印参数配置：默认规则
            WatermarkVO watermarkVO = signatureCommonApi.fetchWatermarkConfig(contractEntity.getContractFileId(), contractEntity.getId(),
                    contractEntity.getBillCode(), PurCommonBillTypeEnum.设备公司设备采购合同.getBillTypeCode(),
                    ContractTypeEnum.设备公司设备采购合同.getTypeCode());
            Assert.notNull(watermarkVO, "获取水印系统参数失败!");
            // 获取上下文并异步调用添加水印
            HttpServletRequest request = ((ServletRequestAttributes) RequestContextHolder.getRequestAttributes()).getRequest();
            HashMap<String, String> headers = new HashMap<>();
            headers.put("authority", request.getHeader("authority"));
            headers.put("ejc-token", request.getHeader("ejc-token"));
            purchaseContractAsyncService.fetchWatermarkAttachment(headers, watermarkVO);
        }
        contractService.saveOrUpdate(contractEntity);
        // 推送项目池
        contractService.pushContract(BeanMapper.map(contractEntity, PurchaseContractVO.class));
        return CommonResponse.success("合同审批回调成功！");
    }

    /**
     * 检查合同文件
     *
     * @param contractEntity
     */
    private void checkContractFile(PurchaseContractEntity contractEntity) {
        //查询合同对饮附件是否存在
        CommonResponse<AttachmentVO> attachResp = attachmentApi.queryDetail(contractEntity.getContractFileId().toString());
        if(!attachResp.isSuccess() || null == attachResp.getData()) {
            logger.info("合同存储文件Id-{}在文件中心不存在，根据参数sourceId-{}, sourceType-{}, billType-{} 查询对应文件信息",
                    contractEntity.getContractFileId(),
                    contractEntity.getId(),
                    ("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "equipPurchaseContract" : "contractFile",
                    PurCommonBillTypeEnum.设备公司设备采购合同.getBillTypeCode());

            //合同附件不存在，根据合同Id、业务类型、查询是否存在对应的文件信息，若存在则更新到合同中，否则抛出异常
            CommonResponse<List<AttachmentVO>> attachListResp = attachmentApi.queryListBySourceId(contractEntity.getId(),
                    PurCommonBillTypeEnum.设备公司设备采购合同.getBillTypeCode(),
                    ("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "equipPurchaseContract" : "contractFile", "desc");

            logger.info("根据参数sourceId-{}, sourceType-{}, billType-{} 查询对应文件信息结果：{}",
                    contractEntity.getId(),
                    ("2".equals(contractEntity.getDraftType()) || "3".equals(contractEntity.getDraftType())) ? "equipPurchaseContract" : "contractFile",
                    PurCommonBillTypeEnum.设备公司设备采购合同.getBillTypeCode(), JSONObject.toJSONString(attachListResp));

            if(attachListResp.isSuccess() || CollectionUtils.isEmpty(attachListResp.getData())) {
                AttachmentVO attachmentVO = attachListResp.getData().get(0);
                logger.info("将合同id-{}对应合同文件Id-{}改为{}", contractEntity.getId(), contractEntity.getContractFileId(), attachmentVO.getId());
                contractEntity.setContractFileId(attachmentVO.getId());
                contractEntity.setContractFilePath(attachmentVO.getTruePath());
            } else {
                throw new BusinessException("审批失败，获取合同文件信息失败, 请检查合同文件！");
            }
        }
    }

    /**
     * 弃审前事件回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> beforeAbstainingProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return bpmBackCheck(billId, state, billTypeCode);
    }

    /**
     * 弃审后事件回调
     *
     * @param billId
     * @param state
     * @return
     */
    @Override
    public CommonResponse<String> afterAbstainingProcessor(Long billId, Integer state, String billTypeCode) {
        //TODO
        return CommonResponse.success();
    }

    private CommonResponse<String> bpmBackCheck(Long billId, Integer state, String billTypeCode) {
		/*
			弃审逻辑：
			1、审批过程中，支持后台弃审；
			2、当审批通过，且没有走签章流程且没有下游业务时，支持后台弃审；
		 */
        PurchaseContractEntity contractEntity = contractService.selectById(billId);

        // 审批可以撤回的签章状态集合
        ArrayList<String> statusList = new ArrayList<>();
        statusList.add(SignatureStatusEnum.未签章.getCode());
        statusList.add(SignatureStatusEnum.已退回.getCode());
        statusList.add(SignatureStatusEnum.已撤回.getCode());
        statusList.add(SignatureStatusEnum.已过期.getCode());

        //使用电子签章且已有签章流程则不允许撤回
        if (DraftTypeEnum.线上起草.getCode().toString().equals(contractEntity.getDraftType())
                || DraftTypeEnum.上传合同.getCode().toString().equals(contractEntity.getDraftType())) {
            if (!statusList.contains(contractEntity.getSignatureStatus())) {
                return CommonResponse.error("当前单据已有签章流程，不能撤回！");
            }
        }
        else if (SignatureStatusEnum.已签章.getCode().equals(contractEntity.getSignatureStatus())) {
            contractEntity.setSignatureStatus(SignatureStatusEnum.未签章.getCode());
            contractEntity.setContractPerformanceState(PerformanceStatusEnum.未签订.getCode());
        }

        contractService.saveOrUpdate(contractEntity);
        //合同是否被其他单据引用
        CommonResponse<String> res = billTypeApi.checkQuote(billTypeCode, billId);
        logger.info("平台返回查询被引用情况" + res.isSuccess() + "----" + res.getMsg());

        if (!res.isSuccess()) {
            return CommonResponse.error("当前单据已被下游业务引用，不能弃审！");
        }

        // 删除文件中心水印文件
        if(contractEntity.getWatermarkContractFileId() != null) {
            boolean delWatermarkflag = contractService.delWatermarkContractFile(contractEntity.getWatermarkContractFileId());
            if (delWatermarkflag) {
                contractEntity.setWatermarkContractFileId(null);
            }
        }
        contractEntity.setContractPerformanceState(PerformanceStatusEnum.未签订.getCode());
        contractEntity.setFilingStatus(FilingStatusEnum.未归档.getTypeCode());
        contractService.saveOrUpdate(contractEntity, false);

        return CommonResponse.success("弃审/撤回前校验回调成功！");
    }

}
